@@ -1,10 +1,10 @@
-__version__ = "1.0.12"
+__version__ = "1.0.13"
 
 from .resolvers.default_resolver import DefaultResolver
 from .resolvers.resolver import Resolver
 from .resolvers.scalar_resolvers.scalar_resolver import ScalarResolver
 from .resolvers.collection_resolvers.collection_resolver import CollectionResolver
-from .resolvers.post_processors.post_processor import PostProcessor
+from .resolvers.processors.processor import Processor
 
 from .functional.load import from_string, from_file_uri, from_cli
 from .functional.util import merge, mask_in, mask_out
