@@ -1,4 +1,4 @@
-from ..entities.wrappers import KCDict, KCList
+from ..entities.wrappers import KCDict, KCList, KCScalar
 from copy import deepcopy
 
 def apply(node, pre_fn=None, post_fn=None, parent_node=None, parent_accessor=None, container=None):
@@ -72,23 +72,59 @@ def select(root_node, accessors):
     return cur_node
 
 def merge(base, to_merge):
-    if base is None:
-        return deepcopy(to_merge)
     base = deepcopy(base)
     to_merge = deepcopy(to_merge)
+    return _merge_fn(dict(root=base), dict(root=to_merge))["root"]
+
+def _merge_fn(base, to_merge):
+    if not isinstance(base, (KCList, list, KCDict, dict)):
+        return to_merge
+    if isinstance(to_merge, (KCList, list)) and not isinstance(base, (KCList, list)):
+        from ..errors import incompatible_type
+        raise incompatible_type(type(base), type(to_merge))
+    if isinstance(to_merge, (KCDict, dict)) and not isinstance(base, (KCDict, dict)):
+        from ..errors import incompatible_type
+        raise incompatible_type(type(base), type(to_merge))
+
+    if isinstance(to_merge, (KCDict, dict)):
+        _merge_dict_fn(base, to_merge)
+    elif isinstance(to_merge, (KCList, list)):
+        _merge_list_fn(base, to_merge)
+    else:
+        return to_merge
+    return base
+
+def _merge_list_fn(base, to_merge):
+    for i in range(len(to_merge)):
+        if i < len(base):
+            base[i] = _merge_fn(base[i], to_merge[i])
+        else:
+            base.append(to_merge[i])
+
+def _merge_dict_fn(base, to_merge):
     for key, value in to_merge.items():
         accessors = string_to_accessors(key)
         node = select(base, accessors[:-1])
+        accessor = accessors[-1]
 
         # allow appending to a list if the last accessor is 'add' or 'append'
-        if isinstance(node, (KCList, list)) and accessors[-1] in ["add", "append"]:
-            if not isinstance(value, (KCList, list)):
-                from ..errors import unexpected_type_error
-                raise unexpected_type_error([KCList, list], value)
-            node += value
+        if isinstance(node, (KCList, list)):
+            if accessor in ["add", "append"]:
+                if not isinstance(value, (KCList, list)):
+                    from ..errors import unexpected_type_error
+                    raise unexpected_type_error([KCList, list], value)
+                node += value
+            elif accessor == "set":
+                parent_node = select(base, accessors[:-2])
+                parent_node[accessors[-2]] = value
+            else:
+                from ..errors import list_merge_invalid_resolving_strategy
+                raise list_merge_invalid_resolving_strategy(key)
         else:
-            node[accessors[-1]] = value
-    return base
+            if accessor in node:
+                node[accessor] = _merge_fn(node[accessor], value)
+            else:
+                node[accessor] = value
 
 def mask_out(dict_, keys_to_mask_out):
     masked_dict = type(dict_)()
